/*
Контроллер, MQTT-клиент для ioBroker, работа через адаптер MQTT (тек. версия 0.2.4)
IP address: 192.168.69.71
Platform: Arduino UNO R3 + EthernetShield W5100
IDE: 1.0.5-r2
Date: 15.05.2015
Version: 0.1
*****
Для тестирования опрашиваются аналоговые входы pinA0, pinA1, pinA2. Значения типа int (АЦП 0...1023).
ID клиента, как и корневая папка "arduinoPLC71".
При старте автоматически создаются топики:
1. "arduinoPLC71/status" - состояние контроллера (online, error)
2. "arduinoPLC71/pollTimer" - интервал публикации переменных (клиент подписан на это значение), секунды
3. "arduinoPLC71/timeout" - интервал проверки на наличие соединения с брокером и, в случае разрыва, реконнект, секунды
4. "arduinoPLC72/buttonХ" - состояние кнопки (4 шт.) - публикация в боркер при изменении состояния (значения "1" или "0")
5. "arduinoPLC72/relayХ" - управление реле (4 шт.) - можно передавать значения "ON", "1" или "true"
При старте настраивается сетевое соединение (MAC-адрес и статический IP адрес), инициализируется отладочный
порт и контроллер подключается к MQTT-брокеру в качестве клиента. Во время работы основного цикла проверяется
состояние соединения с MQTT-боркером, если оно прервано, контроллер пытается его восстановить
Данные публикуются при изменении состояния (кнопки)
**/
//------------------------------------------ Libraries ------------------------------------------//
//Подключаем библиотеки
#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h> //https://github.com/knolleary/pubsubclient
//---------------------------------------- End libraries ----------------------------------------//

//------------------------------------------- Ethernet ------------------------------------------//
//Глобальные константы для работы с сетью
byte mac[]    = {  0xDE, 0xED, 0xBA, 0xFE, 0xFE, 0x72 };
byte mqttserver[] = { 192, 168, 69, 51 };
byte ip[]     = { 192, 168, 69, 72 };
EthernetClient ethClient;
PubSubClient client(mqttserver, 1883, callback, ethClient);
//----------------------------------------- End ethernet ----------------------------------------//

//-------------------------------------------- DI-DO --------------------------------------------//
#define relay1_pin 5
#define relay2_pin 4
#define relay3_pin 3
#define relay4_pin 2
#define button1_pin 9
#define button2_pin 8
#define button3_pin 7
#define button4_pin 6
boolean button1_st;
boolean button2_st;
boolean button3_st;
boolean button4_st;
boolean relay1_st;
boolean relay2_st;
boolean relay3_st;
boolean relay4_st;
boolean led_st;
unsigned long lastTime = 0;
unsigned long poolTime = 60; // по-умолчанию интервал опроса 60 сек
unsigned long timeout = 10; // по-умолчанию таймаут перед восстановлением соединения 10 сек
//------------------------------------------ End DI-DO ------------------------------------------//

//###############################################################################################//
//########################################## Functions ##########################################//
//###############################################################################################//

//------------------------------------------- Callback -----------------------------------------//
//Функция обработки входящих соединений - прием данных по подписке
void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println ("");
  Serial.println ("-------");
  Serial.println ("New callback of MQTT-broker");
  //преобразуем тему(topic) и значение (payload) в строку
  payload[length] = '\0';
  String strTopic = String(topic);
  String strPayload = String((char*)payload);
  //Исследуем что "прилетело" от сервера
  if (strTopic == "arduinoPLC72/timeout") {
    timeout = strPayload.toInt();
  }
  if (strTopic == "arduinoPLC72/pollTimer") {
    poolTime = strPayload.toInt();
  }
  else if (strTopic == "arduinoPLC72/relay1") {
    if (strPayload == "OFF" || strPayload == "0" || strPayload == "false") digitalWrite(relay1_pin, HIGH);
    else if (strPayload == "ON" || strPayload == "1" || strPayload == "true") digitalWrite(relay1_pin, LOW);
  }
  else if (strTopic == "arduinoPLC72/relay2") {
    if (strPayload == "OFF" || strPayload == "0" || strPayload == "false") digitalWrite(relay2_pin, HIGH);
    else if (strPayload == "ON" || strPayload == "1" || strPayload == "true") digitalWrite(relay2_pin, LOW);
  }
  else if (strTopic == "arduinoPLC72/relay3") {
    if (strPayload == "OFF" || strPayload == "0" || strPayload == "false") digitalWrite(relay3_pin, HIGH);
    else if (strPayload == "ON" || strPayload == "1" || strPayload == "true") digitalWrite(relay3_pin, LOW);
  }
  else if (strTopic == "arduinoPLC72/relay4") {
    if (strPayload == "OFF" || strPayload == "0" || strPayload == "false") digitalWrite(relay4_pin, HIGH);
    else if (strPayload == "ON" || strPayload == "1" || strPayload == "true") digitalWrite(relay4_pin, LOW);
  }  
  Serial.print (strTopic);
  Serial.print (" ");
  Serial.println (strPayload);
  Serial.println ("-------");
  Serial.println ("");  
}
//----------------------------------------- End callback ---------------------------------------//

//------------------------------------------ Void setup ----------------------------------------//
void setup() {
  Serial.begin(9600);
  Serial.println("Start mqtt-client controller arduinoPLC72...");
  //стартуем сетевое подключение
  Ethernet.begin(mac, ip);
  Serial.print("PLC's IP: ");
  Serial.println(Ethernet.localIP());
  //Инициализируем порты ввода-вывода (кнопки)
  pinMode(button1_pin, INPUT);
  pinMode(button2_pin, INPUT);
  pinMode(button3_pin, INPUT);
  pinMode(button4_pin, INPUT);
  //Инициализируем порты ввода-вывода (реле), прописываем начальные значения
  pinMode(relay1_pin, OUTPUT);
  pinMode(relay2_pin, OUTPUT);
  pinMode(relay3_pin, OUTPUT);
  pinMode(relay4_pin, OUTPUT);
  digitalWrite(relay1_pin, HIGH);
  digitalWrite(relay2_pin, HIGH);
  digitalWrite(relay3_pin, HIGH);
  digitalWrite(relay4_pin, HIGH);
  Serial.print("Connect to MQTT-boker...  ");
  //подключаемся к MQTT-брокеру
  if (client.connect("arduinoPLC72")) {
    Serial.println("success");
    //Публикуем переменные для отправки данных (статус, кнопки)
    client.publish("arduinoPLC72/status","online");
    if (digitalRead(button1_pin)) {
    client.publish("arduinoPLC72/button1","1");
    } else {
    client.publish("arduinoPLC72/button1","0");
    }
    if (digitalRead(button2_pin)) {
    client.publish("arduinoPLC72/button2","1");
    } else {
    client.publish("arduinoPLC72/button2","0");
    }
    if (digitalRead(button3_pin)) {
    client.publish("arduinoPLC72/button3","1");
    } else {
    client.publish("arduinoPLC72/button3","0");
    }
    if (digitalRead(button4_pin)) {
    client.publish("arduinoPLC72/button4","1");
    } else {
    client.publish("arduinoPLC72/button4","0");
    }
    //Подписываемся на переменные (период опроса, реле)
    client.subscribe("arduinoPLC72/pollTimer");
    client.subscribe("arduinoPLC72/timeout");
    client.subscribe("arduinoPLC72/relay1");
    client.subscribe("arduinoPLC72/relay2");
    client.subscribe("arduinoPLC72/relay3");
    client.subscribe("arduinoPLC72/relay4");
  }
  else {
    Serial.println("failed!");
  }
}
//---------------------------------------- End void setup --------------------------------------//

//------------------------------------------ Void loop -----------------------------------------//
void loop() {
  //Проверка входящих соединений по подписке
  client.loop();
  //Если соединение не активно в течении таймаута, восстанавливаем
  if(millis() > (lastTime + timeout*1000)) {
    lastTime = millis();
    if (!client.connected()) {
      Serial.print("Reconnect to MQTT-boker...  ");
      //пытаемся восстановить
      if (client.connect("arduinoPLC72")) {
        Serial.println("success");
        client.publish("arduinoPLC72/status","online");
        client.subscribe("arduinoPLC72/pollTimer");
        client.subscribe("arduinoPLC72/timeout");
        client.subscribe("arduinoPLC72/relay1");
        client.subscribe("arduinoPLC72/relay2");
        client.subscribe("arduinoPLC72/relay3");
        client.subscribe("arduinoPLC72/relay4");
      }  else {
        Serial.println("failed!");
      }
    }
  }
  //Проверяем состояние кнопок и отправляем измененное состояние на сервер
  if (client.connected()) {
    boolean btn;
    //Срабатывание кнопки button1
    btn = digitalRead(button1_pin);
    if (btn != button1_st) {
      button1_st = btn;
      if (btn) {
        client.publish("arduinoPLC72/button1","1");
        client.publish("common/led","1");
      } else {
        client.publish("arduinoPLC72/button1","0");
      }
    }
    //Срабатывание кнопки button2
    btn = digitalRead(button2_pin);
    if (btn != button2_st) {
      button2_st = btn;
      if (btn) {
        client.publish("arduinoPLC72/button2","1");
        client.publish("common/led","0");
      } else {
        client.publish("arduinoPLC72/button2","0");
      }
    }
    //Срабатывание кнопки button3
    btn = digitalRead(button3_pin);
    if (btn != button3_st) {
      button3_st = btn;
      if (btn) {
        client.publish("arduinoPLC72/button3","1");
      } else {
        client.publish("arduinoPLC72/button3","0");
      }
    }
    //Срабатывание кнопки button4
    btn = digitalRead(button4_pin);
    if (btn != button4_st) {
      button4_st = btn;
      if (btn) {
        client.publish("arduinoPLC72/button4","1");
      } else {
        client.publish("arduinoPLC72/button4","0");
      }
    }
  }
}
//---------------------------------------- End void loop ---------------------------------------//
